import json
import os
import requests
from app_image_generator.plugins.base import BasePlugin


class SpinnakerPlugin(BasePlugin):
    """
    Expects the following environment variables:

    SPINNAKER_PIPELINE_APP_URL: http://spinnaker/gate/pipelines/mylittlepony
    SPINNAKER_USER: basicauthuser
    SPINNAKER_PASSWORD: basicauthpw
    """
    def build_succeeded(self, app, output):
        for image_name, image_id in self.get_images_from_output(output).items():
            requests.post(
                "%s/%s" % (os.environ.get("SPINNAKER_PIPELINE_APP_URL"), image_name),
                data=json.dumps({
                    "type": "manual",
                    "parameters": {"prebaked_ami": image_id},
                    "user": "[circle]"}
                ),
                auth=(os.environ.get("SPINNAKER_USER"), os.environ.get("SPINNAKER_PASSWORD")),
                headers={
                    'Content-Type': 'application/json'
                }
            )


# encoding: utf-8
from setuptools import setup, find_packages
from app_image_generator import version

setup(
    name = 'app-image-generator',
    version = version,
    description = '',
    author = u'Kristian Øllegaard',
    author_email = 'kristian@oellegaard.com',
    zip_safe=False,
    include_package_data = True,
    packages = find_packages(exclude=[]),
    install_requires=['requests'],
    entry_points={
        'console_scripts': [
            'app-image-generator = app_image_generator.cli:main',
        ]
    },
)



import os, requests
from app_image_generator.plugins.base import BasePlugin


class SlackPlugin(BasePlugin):
    def __init__(self):
        self.webhook_url = os.environ.get("SLACK_WEBHOOK_URL")

    def build_succeeded(self, app, output):
        requests.post(self.webhook_url, json={
            "attachments": [{
                "pretext": f"New {app} images available",
                "fields": [
                    {
                        "title": image_name,
                        "value": image_id,
                        "short": True
                    } for image_name, image_id in self.get_images_from_output(output).items()
                ],
            }]
        })

    def build_failed(self, app, output):
        requests.post(self.webhook_url, json={
            "attachments": [{
                "pretext": f"Building {app} failed!!!!",
                "color": "#F35A00",
                "fields": [
                    {
                        "title": image_name,
                        "value": image_id,
                        "short": True
                    } for image_name, image_id in self.get_images_from_output(output).items()
                ],
            }]
        })



from __future__ import print_function
import json
from subprocess import Popen, PIPE, STDOUT
import pprint
from app_image_generator.deployment import Deployment


def run(base_ami, version, revision, git_revision, deployment_file, app, base_ami_name=None, build_job=None,
        build_number=None, files=None, amis=None, verbosity=0, noop=False, install_command=None,
        extra_account_ids=None, builder_type=None, plugins=None, **kwargs):
    if 'packer_bin' in kwargs:
        packer_bin = kwargs.pop('packer_bin')
    else:
        packer_bin = "packer"
    if not files:
        files = []
    deployment = Deployment(base_ami, version, revision, git_revision, deployment_file, app, base_ami_name, build_job,
                            build_number, files=files, amis=amis, install_command=install_command,
                            extra_account_ids=extra_account_ids, instance_type=kwargs.get('build_instance_type'),
                            builder_type=builder_type
    )
    packer_input = deployment.packer_json()
    if verbosity and verbosity > 0:
        print("PACKER JSON")
        print("===========")
        pprint.pprint(json.loads(packer_input))
        print("===========")
    packer_cmd = [packer_bin, "build", "-"]
    if not noop:
        p = Popen(packer_cmd, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
        output = b''
        try:
            p.stdin.write(str.encode(packer_input))
            p.stdin.close()
            for line in iter(p.stdout.readline, b''):
                output += line
                try:
                    print(line, flush=True)  # Don't add a line break, as it's already provided by the output
                except TypeError:  # Python<3.5
                    print(line)
            p.stdout.close()
            return_code = p.wait()

        except KeyboardInterrupt:
            p.kill()
            return_code = 1

        for plugin in plugins:
            if return_code == 0:
                plugin.build_succeeded(app, output)
            else:
                plugin.build_failed(app, output)

        return return_code
    else:
        print("NOOP: ",  " ".join(packer_cmd))
